@@ -122,7 +122,7 @@
 
 #define SASL_VERSION_MAJOR 2
 #define SASL_VERSION_MINOR 1
-#define SASL_VERSION_STEP 0
+#define SASL_VERSION_STEP 1
 
 #include "prop.h"
 #define LIBSASL_API
@@ -281,6 +281,8 @@ typedef unsigned sasl_ssf_t;
  * PASS_CREDENTIALS     -- require mechanisms which pass client
  *			   credentials, and allow mechanisms which can pass
  *			   credentials to do so
+ * MUTUAL_AUTH          -- require mechanisms which provide mutual
+ *			   authentication
  */
 #define SASL_SEC_NOPLAINTEXT      0x0001
 #define SASL_SEC_NOACTIVE         0x0002
@@ -288,7 +290,8 @@ typedef unsigned sasl_ssf_t;
 #define SASL_SEC_FORWARD_SECRECY  0x0008
 #define SASL_SEC_NOANONYMOUS      0x0010
 #define SASL_SEC_PASS_CREDENTIALS 0x0020
-#define SASL_SEC_MAXIUMUM         0x00FF
+#define SASL_SEC_MUTUAL_AUTH      0x0040
+#define SASL_SEC_MAXIMUM          0x00FF
 
 typedef struct sasl_security_properties 
 { 
@@ -888,7 +891,7 @@ LIBSASL_API int sasl_server_new(const char *service,
  * which lists all possible mechanisms that the library can supply
  *
  * Returns NULL on failure. */
-LIBSASL_API const char ** sasl_global_listmech();
+LIBSASL_API const char ** sasl_global_listmech(void);
 
 /* This returns a list of mechanisms in a NUL-terminated string
  *  conn          -- the connection to list mechanisms for (either client
