@@ -17,7 +17,7 @@
 #include "prop.h"
 #endif
 
-/* callback to lookup a sasl_callback_t for a connnection
+/* callback to lookup a sasl_callback_t for a connection
  * input:
  *  conn        -- the connection to lookup a callback for
  *  callbacknum -- the number of the callback
@@ -155,7 +155,7 @@ typedef struct sasl_utils {
 /*
  * output parameters from SASL API
  *
- * created / destroyed by the glue code, though probabally filled in
+ * created / destroyed by the glue code, though probably filled in
  * by a combination of the plugin, the glue code, and the canon_user callback.
  *
  */
@@ -270,7 +270,9 @@ typedef struct sasl_client_params {
     int spare_int1;
     int spare_int2;
     int spare_int3;
-    int spare_int4;
+
+    /* flags field as passed to sasl_client_new */
+    unsigned flags;
 
     /* set to 0 initially, this allows a plugin with extended parameters
      * to work with an older framework by updating version as parameters
@@ -281,11 +283,26 @@ typedef struct sasl_client_params {
 
 /* features shared between client and server */
 /* These allow the glue code to handle client-first and server-last issues */
+
+/* This indicates that the mechanism prefers to do client-send-first
+ * if the protocol allows it. */
 #define SASL_FEAT_WANT_CLIENT_FIRST 0x0002
-#define SASL_FEAT_WANT_SERVER_LAST 0x0004
-/* This says that client-first is handled internally (e.g. DIGEST-MD5) */
-/* Note that this is not the preferred way of doing things */
-#define SASL_FEAT_INTERNAL_CLIENT_FIRST 0x0008
+
+/* This feature is deprecated, instead, plugins should set *serverout to
+ * non-NULL and return SASL_OK intelligently to allow flexible use of
+ * server-last semantics */
+/* #define SASL_FEAT_WANT_SERVER_LAST 0x0004 */
+
+/* This feature is deprecated, instead plugins should correctly set
+ * SASL_FEAT_SERVER_FIRST as needed */
+/* #define SASL_FEAT_INTERNAL_CLIENT_FIRST 0x0008 */
+
+/* This indicates that the plugin is server-first only. 
+ * Not defining either of SASL_FEAT_SERVER_FIRST or 
+ * SASL_FEAT_WANT_CLIENT_FIRST indicates that the mechanism will take care 
+ * of the client-first situation internally.
+ */
+#define SASL_FEAT_SERVER_FIRST 0x0010
 
 /* client plug-in features */
 #define SASL_FEAT_NEEDSERVERFQDN 0x0001
@@ -497,8 +514,8 @@ typedef struct sasl_server_params {
     int spare_int2;
     int spare_int3;
 
-    /* secflags field as passed to sasl_server_new */
-    unsigned secflags;
+    /* flags field as passed to sasl_server_new */
+    unsigned flags;
 
     /* set to 0 initially, this allows a plugin with extended parameters
      * to work with an older framework by updating version as parameters
