@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: client.c,v 1.40 2002/01/10 22:13:45 rjs3 Exp $
+ * $Id: client.c,v 1.44 2002/01/22 19:51:32 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -277,6 +277,8 @@ int sasl_client_new(const char *service,
   int result;
   sasl_client_conn_t *conn;
   sasl_utils_t *utils;
+
+  if(_sasl_client_active==0) return SASL_NOTINIT;
   
   /* Remember, iplocalport and ipremoteport can be NULL and be valid! */
   if (!pconn || !service || !serverFQDN)
@@ -314,6 +316,7 @@ int sasl_client_new(const char *service,
   utils->conn= *pconn;
   conn->cparams->utils = utils;
   conn->cparams->canon_user = &_sasl_canon_user;
+  conn->cparams->flags = flags;
   
   result = _sasl_strdup(serverFQDN, &conn->serverFQDN, NULL);
   if(result == SASL_OK) return SASL_OK;
@@ -391,6 +394,8 @@ int sasl_client_start(sasl_conn_t *conn,
     sasl_ssf_t bestssf = 0, minssf = 0;
     int result;
 
+    if(_sasl_client_active==0) return SASL_NOTINIT;
+
     if (!conn) return SASL_BADPARAM;
 
     /* verify parameters */
@@ -507,19 +512,15 @@ int sasl_client_start(sasl_conn_t *conn,
 
     /* do a step -- but only if we can do a client-send-first */
  dostep:
-    if(c_conn->mech->plug->features & SASL_FEAT_INTERNAL_CLIENT_FIRST) {
-	/* The plugin handles client-first internally */
-	result = sasl_client_step(conn, NULL, 0, prompt_need,
-				  clientout, clientoutlen);
-    } else if(clientout) {
-	if(c_conn->mech->plug->features & SASL_FEAT_WANT_CLIENT_FIRST) {
-	    result = sasl_client_step(conn, NULL, 0, prompt_need,
-				      clientout, clientoutlen);
-	} else {
-	    *clientout = NULL;
-	    *clientoutlen = 0;
-	    result = SASL_CONTINUE;
-	}
+    if(clientout) {
+        if(c_conn->mech->plug->features & SASL_FEAT_SERVER_FIRST) {
+            *clientout = NULL;
+            *clientoutlen = 0;
+            result = SASL_CONTINUE;
+        } else {
+            result = sasl_client_step(conn, NULL, 0, prompt_need,
+                                      clientout, clientoutlen);
+        }
     }
     else
 	result = SASL_CONTINUE;
@@ -558,6 +559,9 @@ int sasl_client_step(sasl_conn_t *conn,
   if ((serverin==NULL) && (serverinlen>0))
       PARAMERROR(conn);
 
+  if(clientout) *clientout = NULL;
+  if(clientoutlen) *clientoutlen = 0;
+
   /* do a step */
   result = c_conn->mech->plug->mech_step(conn->context,
 					 c_conn->cparams,
@@ -572,8 +576,7 @@ int sasl_client_step(sasl_conn_t *conn,
        * 1. the mech does server-send-last
        * 2. the protocol does not
        * we need to return no data */
-      if(!(conn->flags & SASL_SUCCESS_DATA)
-	 && (c_conn->mech->plug->features & SASL_FEAT_WANT_SERVER_LAST)) {
+      if(!*clientout && !(conn->flags & SASL_SUCCESS_DATA)) {
 	  *clientout = "";
 	  *clientoutlen = 0;
       }
@@ -587,8 +590,7 @@ int sasl_client_step(sasl_conn_t *conn,
 			"mech did not call canon_user for both authzid and authid");
 	  result = SASL_BADPROT;
       }
-  }
-  
+  }  
 
   RETURN(conn,result);
 }
