@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.95 2002/01/10 22:13:46 rjs3 Exp $
+ * $Id: server.c,v 1.101 2002/01/31 17:58:36 ken3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -495,6 +495,7 @@ struct secflag_map_s secflag_map[] = {
     { "forward_secrecy", SASL_SEC_FORWARD_SECRECY },
     { "noanonymous", SASL_SEC_NOANONYMOUS },
     { "pass_credentials", SASL_SEC_PASS_CREDENTIALS },
+    { "mutual_auth", SASL_SEC_MUTUAL_AUTH },
     { NULL, 0x0 }
 };
 
@@ -799,6 +800,7 @@ int sasl_server_new(const char *service,
   serverconn->sparams->canon_user = &_sasl_canon_user;
   serverconn->sparams->serverFQDN = (*pconn)->serverFQDN;
   serverconn->sparams->props = serverconn->base.props;
+  serverconn->sparams->flags = flags;
 
   /* set some variables */
   if (user_realm) {
@@ -833,7 +835,7 @@ static int mech_permitted(sasl_conn_t *conn,
     void *context;
     sasl_ssf_t minssf = 0;
 
-    if(!conn) return SASL_BADPARAM;
+    if(!conn) return 0;
 
     if(! mech || ! mech->plug) {
 	PARAMERROR(conn);
@@ -879,7 +881,7 @@ static int mech_permitted(sasl_conn_t *conn,
     for(cur = s_conn->mech_contexts; cur; cur=cur->next) {
 	if(cur->mech == mech) {
 	    /* If it's not mech_avail'd, then stop now */
-	    if(!cur->context) return SASL_NOMECH;
+	    if(!cur->context) return 0;
 	    break;
 	}
     }
@@ -1047,6 +1049,9 @@ int sasl_server_start(sasl_conn_t *conn,
     if (!mech || ((clientin==NULL) && (clientinlen>0)))
 	PARAMERROR(conn);
 
+    if(serverout) *serverout = NULL;
+    if(serveroutlen) *serveroutlen = 0;
+
     while (m!=NULL)
     {
 	if ( strcasecmp(mech,m->plug->mech_name)==0)
@@ -1156,37 +1161,32 @@ int sasl_server_start(sasl_conn_t *conn,
     }
     
     if (result == SASL_OK) {
-	if(clientin &&
-	   (s_conn->mech->plug->features &
-	    (SASL_FEAT_WANT_CLIENT_FIRST | SASL_FEAT_INTERNAL_CLIENT_FIRST))) {
-	    /* both protocol & mech did client-send-first */
-	    result = sasl_server_step(conn,
-				      clientin, clientinlen,
-				      serverout, serveroutlen);
-	} else if (clientin) {
-	    /* Remote sent first, but mechanism does not support it.
-	     * RFC 2222 says we fail at this point. */
-	    sasl_seterror(conn, 0,
-			  "Remote sent first but mech does not allow it.");
-	    result = SASL_BADPROT;
-	} else {
-	    /* protocol CAN do client-send-first, but no data arrived */
-	    /* OR protocol can't do client-send-first */
-	    if(!(s_conn->mech->plug->features & SASL_FEAT_WANT_CLIENT_FIRST) ||
-	    (s_conn->mech->plug->features & SASL_FEAT_INTERNAL_CLIENT_FIRST)) {
-		/* Mech wants server-first, so let them have it */
-		/* or the send-first situation is handled internally
-		 *  by the mechanism */
-		result = sasl_server_step(conn,
-					  clientin, clientinlen,
-					  serverout, serveroutlen);
-	    } else {
-		/* Mech wants client first anyway, so we should do that */
-		*serverout = "";
-		*serveroutlen = 0;
-		result = SASL_CONTINUE;
-	    }
-	}	 
+         if(clientin) {
+            if(s_conn->mech->plug->features & SASL_FEAT_SERVER_FIRST) {
+                /* Remote sent first, but mechanism does not support it.
+                 * RFC 2222 says we fail at this point. */
+                sasl_seterror(conn, 0,
+                              "Remote sent first but mech does not allow it.");
+                result = SASL_BADPROT;
+            } else {
+                /* Mech wants client-first, so let them have it */
+                result = sasl_server_step(conn,
+                                          clientin, clientinlen,
+                                          serverout, serveroutlen);
+            }
+        } else {
+            if(s_conn->mech->plug->features & SASL_FEAT_WANT_CLIENT_FIRST) {
+                /* Mech wants client first anyway, so we should do that */
+                *serverout = "";
+                *serveroutlen = 0;
+                result = SASL_CONTINUE;
+            } else {
+                /* Mech wants server-first, so let them have it */
+                result = sasl_server_step(conn,
+                                          clientin, clientinlen,
+                                          serverout, serveroutlen);
+            }
+	}
     }
 
  done:
@@ -1240,6 +1240,9 @@ int sasl_server_step(sasl_conn_t *conn,
 	return SASL_OK;
     }
 
+    if(serverout) *serverout = NULL;
+    if(serveroutlen) *serveroutlen = 0;
+
     ret = s_conn->mech->plug->mech_step(conn->context,
 					s_conn->sparams,
 					clientin,
@@ -1259,8 +1262,7 @@ int sasl_server_step(sasl_conn_t *conn,
 	 *
 	 * in this case, return SASL_CONTINUE and remember we are done.
 	 */
-	if(!(conn->flags & SASL_SUCCESS_DATA)
-	   && (s_conn->mech->plug->features & SASL_FEAT_WANT_SERVER_LAST)) {
+	if(*serverout && !(conn->flags & SASL_SUCCESS_DATA)) {
 	    s_conn->sent_last = 1;
 	    ret = SASL_CONTINUE;
 	}
