@@ -1,7 +1,7 @@
 /* Kerberos4 SASL plugin
  * Rob Siemborski
  * Tim Martin 
- * $Id: kerberos4.c,v 1.72 2002/01/09 19:14:17 rjs3 Exp $
+ * $Id: kerberos4.c,v 1.75 2002/01/31 17:58:37 ken3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -897,8 +897,9 @@ static sasl_server_plug_t kerberosv4_server_plugins[] =
   {
     "KERBEROS_V4",
     KRB_DES_SECURITY_BITS,
-    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOACTIVE | SASL_SEC_NOANONYMOUS,
-    0,
+    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOACTIVE | SASL_SEC_NOANONYMOUS
+    | SASL_SEC_MUTUAL_AUTH,
+    SASL_FEAT_SERVER_FIRST,
     NULL,
     &kerberosv4_server_mech_new,
     &kerberosv4_server_mech_step,
@@ -974,7 +975,16 @@ static int kerberosv4_client_mech_new(
 		 sasl_client_params_t *params,
 		 void **conn)
 {
-  return new_text(params->utils, (context_t **) conn);
+    if(!krb_mutex) {
+	krb_mutex = params->utils->mutex_alloc();
+	if(!krb_mutex) {
+	    params->utils->seterror(params->utils->conn, 0,
+				     "couldn't allocate mutex");
+	    return SASL_FAIL;
+	}
+    }
+
+    return new_text(params->utils, (context_t **) conn);
 }
 
 static int kerberosv4_client_mech_step(void *conn_context,
@@ -1427,8 +1437,9 @@ static sasl_client_plug_t kerberosv4_client_plugins[] =
   {
     "KERBEROS_V4",
     KRB_DES_SECURITY_BITS,
-    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOACTIVE | SASL_SEC_NOANONYMOUS,
-    SASL_FEAT_NEEDSERVERFQDN,
+    SASL_SEC_NOPLAINTEXT | SASL_SEC_NOACTIVE | SASL_SEC_NOANONYMOUS
+    | SASL_SEC_MUTUAL_AUTH,
+    SASL_FEAT_NEEDSERVERFQDN | SASL_FEAT_SERVER_FIRST,
     kerberosv4_client_required_prompts,
     NULL,
     &kerberosv4_client_mech_new,
